@@ -1,6 +1,6 @@
 from sympy import symbols, sin, exp, cos, Derivative, Integral, Basic, \
     count_ops, S, And, I, pi, Eq, Or, Not, Xor, Nand, Nor, Implies, \
-    Equivalent, MatrixSymbol, Symbol, ITE, Rel
+    Equivalent, MatrixSymbol, Symbol, ITE, Rel, Rational, Sum
 from sympy.core.containers import Tuple
 
 x, y, z = symbols('x,y,z')
@@ -30,11 +30,11 @@ def test_count_ops_non_visual():
 
 
 def test_count_ops_visual():
-    ADD, MUL, POW, SIN, COS, EXP, AND, D, G = symbols(
-        'Add Mul Pow sin cos exp And Derivative Integral'.upper())
+    ADD, MUL, POW, SIN, COS, EXP, AND, D, G, M = symbols(
+        'Add Mul Pow sin cos exp And Derivative Integral Sum'.upper())
     DIV, SUB, NEG = symbols('DIV SUB NEG')
     LT, LE, GT, GE, EQ, NE = symbols('LT LE GT GE EQ NE')
-    NOT, OR, AND, XOR, IMPLIES, EQUIVALENT, ITE, BASIC, TUPLE = symbols(
+    NOT, OR, AND, XOR, IMPLIES, EQUIVALENT, _ITE, BASIC, TUPLE = symbols(
         'Not Or And Xor Implies Equivalent ITE Basic Tuple'.upper())
 
     def count(val):
@@ -45,6 +45,7 @@ def test_count_ops_visual():
     assert count(-1) == NEG
     assert count(-2) == NEG
     assert count(S(2)/3) == DIV
+    assert count(Rational(2, 3)) == DIV
     assert count(pi/3) == DIV
     assert count(-pi/3) == DIV + NEG
     assert count(I - 1) == SUB
@@ -54,6 +55,7 @@ def test_count_ops_visual():
     assert count(x) is S.Zero
     assert count(-x) == NEG
     assert count(-2*x/3) == NEG + DIV + MUL
+    assert count(Rational(-2, 3)*x) == NEG + DIV + MUL
     assert count(1/x) == DIV
     assert count(1/(x*y)) == DIV + MUL
     assert count(-1/x) == NEG + DIV
@@ -66,7 +68,8 @@ def test_count_ops_visual():
     assert count(-2*x**2) == POW + MUL + NEG
 
     assert count(x + pi/3) == ADD + DIV
-    assert count(x + S(1)/3) == ADD + DIV
+    assert count(x + S.One/3) == ADD + DIV
+    assert count(x + Rational(1, 3)) == ADD + DIV
     assert count(x + y) == ADD
     assert count(x - y) == SUB
     assert count(y - x) == SUB
@@ -85,6 +88,7 @@ def test_count_ops_visual():
 
     assert count(Derivative(x, x)) == D
     assert count(Integral(x, x) + 2*x/(1 + x)) == G + DIV + MUL + 2*ADD
+    assert count(Sum(x, (x, 1, x + 1)) + 2*x/(1 + x)) == M + DIV + MUL + 3*ADD
     assert count(Basic()) is S.Zero
 
     assert count({x + 1: sin(x)}) == ADD + SIN
@@ -107,7 +111,7 @@ def test_count_ops_visual():
     assert count(Xor(x, y)) == XOR
     assert count(Implies(x, y)) == IMPLIES
     assert count(Equivalent(x, y)) == EQUIVALENT
-    assert count(ITE(x, y, z)) == ITE
+    assert count(ITE(x, y, z)) == _ITE
     assert count([Or(x, y), And(x, y), Basic(x + y)]
         ) == ADD + AND + BASIC + OR
 
