@@ -6,16 +6,14 @@ Features:
   - VSIDS heuristic
 
 References:
-  - http://en.wikipedia.org/wiki/DPLL_algorithm
+  - https://en.wikipedia.org/wiki/DPLL_algorithm
 """
-from __future__ import print_function, division
 
 from collections import defaultdict
 from heapq import heappush, heappop
 
-from sympy.core.compatibility import range
-from sympy import default_sort_key, ordered
-from sympy.logic.boolalg import conjuncts, to_cnf, to_int_repr, _find_predicates
+from sympy import ordered
+from sympy.assumptions.cnf import EncodedCNF
 
 
 def dpll_satisfiable(expr, all_models=False):
@@ -35,16 +33,18 @@ def dpll_satisfiable(expr, all_models=False):
     False
 
     """
-    clauses = conjuncts(to_cnf(expr))
-    if False in clauses:
+    if not isinstance(expr, EncodedCNF):
+        exprs = EncodedCNF()
+        exprs.add_prop(expr)
+        expr = exprs
+
+    # Return UNSAT when False (encoded as 0) is present in the CNF
+    if {0} in expr.data:
         if all_models:
             return (f for f in [False])
         return False
-    symbols = sorted(_find_predicates(expr), key=default_sort_key)
-    symbols_int_repr = range(1, len(symbols) + 1)
-    clauses_int_repr = to_int_repr(clauses, symbols)
 
-    solver = SATSolver(clauses_int_repr, symbols_int_repr, set(), symbols)
+    solver = SATSolver(expr.data, expr.variables, set(), expr.symbols)
     models = solver._find_model()
 
     if all_models:
@@ -72,7 +72,7 @@ def _all_models(models):
             yield False
 
 
-class SATSolver(object):
+class SATSolver:
     """
     Class for representing a SAT solver capable of
      finding a model to a boolean theory in conjunctive
@@ -173,16 +173,17 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
         >>> list(l._find_model())
         [{1: True, 2: False, 3: False}, {1: True, 2: True, 3: True}]
 
         >>> from sympy.abc import A, B, C
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]), [A, B, C])
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set(), [A, B, C])
         >>> list(l._find_model())
         [{A: True, B: False, C: False}, {A: True, B: True, C: True}]
+
         """
 
         # We use this variable to keep track of if we should flip a
@@ -213,8 +214,8 @@ class SATSolver(object):
 
                 # Stopping condition for a satisfying theory
                 if 0 == lit:
-                    yield dict((self.symbols[abs(lit) - 1],
-                                lit > 0) for lit in self.var_settings)
+                    yield {self.symbols[abs(lit) - 1]:
+                                lit > 0 for lit in self.var_settings}
                     while self._current_level.flipped:
                         self._undo()
                     if len(self.levels) == 1:
@@ -267,7 +268,7 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([1]), set([2])], set([1, 2]), set([]))
+        >>> l = SATSolver([{1}, {2}], {1, 2}, set())
         >>> next(l._find_model())
         {1: True, 2: True}
         >>> l._current_level.decision
@@ -275,7 +276,8 @@ class SATSolver(object):
         >>> l._current_level.flipped
         False
         >>> l._current_level.var_settings
-        set([1, 2])
+        {1, 2}
+
         """
         return self.levels[-1]
 
@@ -286,7 +288,7 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([1]), set([-1])], set([1]), set([]))
+        >>> l = SATSolver([{1}, {-1}], {1}, set())
         >>> try:
         ...     next(l._find_model())
         ... except StopIteration:
@@ -295,6 +297,7 @@ class SATSolver(object):
         False
         >>> l._clause_sat(1)
         True
+
         """
         for lit in self.clauses[cls]:
             if lit in self.var_settings:
@@ -308,14 +311,15 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
         >>> next(l._find_model())
         {1: True, 2: False, 3: False}
         >>> l._is_sentinel(2, 3)
         True
         >>> l._is_sentinel(-3, 1)
         False
+
         """
         return cls in self.sentinels[lit]
 
@@ -332,22 +336,23 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
         >>> next(l._find_model())
         {1: True, 2: False, 3: False}
         >>> l.var_settings
-        set([-3, -2, 1])
+        {-3, -2, 1}
 
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
         >>> l._assign_literal(-1)
         >>> try:
         ...     next(l._find_model())
         ... except StopIteration:
         ...     pass
         >>> l.var_settings
-        set([-1])
+        {-1}
+
         """
         self.var_settings.add(lit)
         self._current_level.var_settings.add(lit)
@@ -381,17 +386,18 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
         >>> next(l._find_model())
         {1: True, 2: False, 3: False}
         >>> level = l._current_level
         >>> level.decision, level.var_settings, level.flipped
-        (-3, set([-3, -2]), False)
+        (-3, {-3, -2}, False)
         >>> l._undo()
         >>> level = l._current_level
         >>> level.decision, level.var_settings, level.flipped
-        (0, set([1]), False)
+        (0, {1}, False)
+
         """
         # Undo the variable settings
         for lit in self._current_level.var_settings:
@@ -417,20 +423,21 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
         >>> l.variable_set
         [False, False, False, False]
         >>> l.sentinels
-        {-3: set([0, 2]), -2: set([3, 4]), 2: set([0, 3]), 3: set([2, 4])}
+        {-3: {0, 2}, -2: {3, 4}, 2: {0, 3}, 3: {2, 4}}
 
         >>> l._simplify()
 
         >>> l.variable_set
         [False, True, False, False]
         >>> l.sentinels
-        {-3: set([0, 2]), -2: set([3, 4]), -1: set(), 2: set([0, 3]),
-        ...3: set([2, 4])}
+        {-3: {0, 2}, -2: {3, 4}, -1: set(), 2: {0, 3},
+        ...3: {2, 4}}
+
         """
         changed = True
         while changed:
@@ -477,8 +484,8 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
 
         >>> l.lit_scores
         {-3: -2.0, -2: -2.0, -1: 0.0, 1: 0.0, 2: -2.0, 3: -2.0}
@@ -487,6 +494,7 @@ class SATSolver(object):
 
         >>> l.lit_scores
         {-3: -1.0, -2: -1.0, -1: 0.0, 1: 0.0, 2: -1.0, 3: -1.0}
+
         """
         # We divide every literal score by 2 for a decay factor
         #  Note: This doesn't change the heap property
@@ -501,8 +509,8 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
 
         >>> l.lit_heap
         [(-2.0, -3), (-2.0, 2), (-2.0, -2), (0.0, 1), (-2.0, 3), (0.0, -1)]
@@ -512,6 +520,7 @@ class SATSolver(object):
 
         >>> l.lit_heap
         [(-2.0, -2), (-2.0, 2), (0.0, -1), (0.0, 1), (-2.0, 3)]
+
         """
         if len(self.lit_heap) == 0:
             return 0
@@ -535,8 +544,8 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
         >>> l.lit_heap
         [(-2.0, -3), (-2.0, 2), (-2.0, -2), (0.0, 1), (-2.0, 3), (0.0, -1)]
 
@@ -545,6 +554,7 @@ class SATSolver(object):
         >>> l.lit_heap
         [(-2.0, -3), (-2.0, -2), (-2.0, -2), (-2.0, 2), (-2.0, 3), (0.0, -1),
         ...(-2.0, 2), (0.0, 1)]
+
         """
         var = abs(lit)
         heappush(self.lit_heap, (self.lit_scores[var], var))
@@ -557,20 +567,21 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
 
         >>> l.num_learned_clauses
         0
         >>> l.lit_scores
         {-3: -2.0, -2: -2.0, -1: 0.0, 1: 0.0, 2: -2.0, 3: -2.0}
 
-        >>> l._vsids_clause_added(set([2, -3]))
+        >>> l._vsids_clause_added({2, -3})
 
         >>> l.num_learned_clauses
         1
         >>> l.lit_scores
         {-3: -1.0, -2: -2.0, -1: 0.0, 1: 0.0, 2: -1.0, 3: -2.0}
+
         """
         self.num_learned_clauses += 1
         for lit in cls:
@@ -586,22 +597,23 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
 
         >>> l.num_learned_clauses
         0
         >>> l.clauses
         [[2, -3], [1], [3, -3], [2, -2], [3, -2]]
         >>> l.sentinels
-        {-3: set([0, 2]), -2: set([3, 4]), 2: set([0, 3]), 3: set([2, 4])}
+        {-3: {0, 2}, -2: {3, 4}, 2: {0, 3}, 3: {2, 4}}
 
         >>> l._simple_add_learned_clause([3])
 
         >>> l.clauses
         [[2, -3], [1], [3, -3], [2, -2], [3, -2], [3]]
         >>> l.sentinels
-        {-3: set([0, 2]), -2: set([3, 4]), 2: set([0, 3]), 3: set([2, 4, 5])}
+        {-3: {0, 2}, -2: {3, 4}, 2: {0, 3}, 3: {2, 4, 5}}
+
         """
         cls_num = len(self.clauses)
         self.clauses.append(cls)
@@ -622,12 +634,13 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
-        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> l = SATSolver([{2, -3}, {1}, {3, -3}, {2, -2},
+        ... {3, -2}], {1, 2, 3}, set())
         >>> next(l._find_model())
         {1: True, 2: False, 3: False}
         >>> l._simple_compute_conflict()
         [3]
+
         """
         return [-(level.decision) for level in self.levels[1:]]
 
@@ -636,7 +649,7 @@ class SATSolver(object):
         pass
 
 
-class Level(object):
+class Level:
     """
     Represents a single level in the DPLL algorithm, and contains
     enough information for a sound backtracking procedure.
