@@ -7,7 +7,7 @@ from sympy.strategies.traverse import bottom_up
 from sympy.utilities import sift
 from sympy.utilities.misc import filldedent
 
-from sympy.matrices.expressions.matexpr import MatrixExpr, ZeroMatrix, Identity, MatrixElement
+from sympy.matrices.expressions.matexpr import MatrixExpr, MatrixElement
 from sympy.matrices.expressions.matmul import MatMul
 from sympy.matrices.expressions.matadd import MatAdd
 from sympy.matrices.expressions.matpow import MatPow
@@ -16,6 +16,7 @@ from sympy.matrices.expressions.trace import trace
 from sympy.matrices.expressions.determinant import det, Determinant
 from sympy.matrices.expressions.slice import MatrixSlice
 from sympy.matrices.expressions.inverse import Inverse
+from sympy.matrices.expressions.special import ZeroMatrix, Identity
 from sympy.matrices import Matrix, ShapeError
 from sympy.functions.elementary.complexes import re, im
 
@@ -219,7 +220,7 @@ class BlockMatrix(MatrixExpr):
         ========
 
         >>> from sympy import MatrixSymbol, BlockMatrix, ZeroMatrix
-        >>> from sympy.abc import l, m, n
+        >>> from sympy.abc import m, n
         >>> X = MatrixSymbol('X', n, n)
         >>> Y = MatrixSymbol('Y', m ,m)
         >>> Z = MatrixSymbol('Z', n, m)
@@ -287,7 +288,7 @@ class BlockDiagMatrix(BlockMatrix):
     Examples
     ========
 
-    >>> from sympy import MatrixSymbol, BlockDiagMatrix, symbols, Identity
+    >>> from sympy import MatrixSymbol, BlockDiagMatrix, symbols
     >>> n, m, l = symbols('n m l')
     >>> X = MatrixSymbol('X', n, n)
     >>> Y = MatrixSymbol('Y', m ,m)
@@ -408,7 +409,7 @@ def block_collapse(expr):
     """Evaluates a block matrix expression
 
     >>> from sympy import MatrixSymbol, BlockMatrix, symbols, \
-                          Identity, Matrix, ZeroMatrix, block_collapse
+                          Identity, ZeroMatrix, block_collapse
     >>> n,m,l = symbols('n m l')
     >>> X = MatrixSymbol('X', n, n)
     >>> Y = MatrixSymbol('Y', m ,m)
