@@ -1,12 +1,10 @@
 from sympy.core.backend import sympify, Add, ImmutableMatrix as Matrix
-from sympy.core.compatibility import unicode
-from .printing import (VectorLatexPrinter, VectorPrettyPrinter,
-                       VectorStrPrinter)
+from sympy.printing.defaults import Printable
 
 __all__ = ['Dyadic']
 
 
-class Dyadic(object):
+class Dyadic(Printable):
     """A Dyadic object.
 
     See:
@@ -98,12 +96,10 @@ class Dyadic(object):
                 ol += v[0] * v[1] * (v[2] & other)
         return ol
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         """Divides the Dyadic by a sympifyable expression. """
         return self.__mul__(1 / other)
 
-    __truediv__ = __div__
-
     def __eq__(self, other):
         """Tests for equality.
 
@@ -152,27 +148,26 @@ class Dyadic(object):
     def __neg__(self):
         return self * -1
 
-    def _latex(self, printer=None):
+    def _latex(self, printer):
         ar = self.args  # just to shorten things
         if len(ar) == 0:
             return str(0)
         ol = []  # output list, to be concatenated to a string
-        mlp = VectorLatexPrinter()
         for i, v in enumerate(ar):
             # if the coef of the dyadic is 1, we skip the 1
             if ar[i][0] == 1:
-                ol.append(' + ' + mlp.doprint(ar[i][1]) + r"\otimes " +
-                          mlp.doprint(ar[i][2]))
+                ol.append(' + ' + printer._print(ar[i][1]) + r"\otimes " +
+                          printer._print(ar[i][2]))
             # if the coef of the dyadic is -1, we skip the 1
             elif ar[i][0] == -1:
                 ol.append(' - ' +
-                          mlp.doprint(ar[i][1]) +
+                          printer._print(ar[i][1]) +
                           r"\otimes " +
-                          mlp.doprint(ar[i][2]))
+                          printer._print(ar[i][2]))
             # If the coefficient of the dyadic is not 1 or -1,
             # we might wrap it in parentheses, for readability.
             elif ar[i][0] != 0:
-                arg_str = mlp.doprint(ar[i][0])
+                arg_str = printer._print(ar[i][0])
                 if isinstance(ar[i][0], Add):
                     arg_str = '(%s)' % arg_str
                 if arg_str.startswith('-'):
@@ -180,8 +175,8 @@ class Dyadic(object):
                     str_start = ' - '
                 else:
                     str_start = ' + '
-                ol.append(str_start + arg_str + mlp.doprint(ar[i][1]) +
-                          r"\otimes " + mlp.doprint(ar[i][2]))
+                ol.append(str_start + arg_str + printer._print(ar[i][1]) +
+                          r"\otimes " + printer._print(ar[i][2]))
         outstr = ''.join(ol)
         if outstr.startswith(' + '):
             outstr = outstr[3:]
@@ -189,7 +184,7 @@ class Dyadic(object):
             outstr = outstr[1:]
         return outstr
 
-    def _pretty(self, printer=None):
+    def _pretty(self, printer):
         e = self
 
         class Fake(object):
@@ -197,29 +192,22 @@ class Dyadic(object):
 
             def render(self, *args, **kwargs):
                 ar = e.args  # just to shorten things
-                settings = printer._settings if printer else {}
-                if printer:
-                    use_unicode = printer._use_unicode
-                else:
-                    from sympy.printing.pretty.pretty_symbology import (
-                        pretty_use_unicode)
-                    use_unicode = pretty_use_unicode()
-                mpp = printer if printer else VectorPrettyPrinter(settings)
+                mpp = printer
                 if len(ar) == 0:
-                    return unicode(0)
-                bar = u"\N{CIRCLED TIMES}" if use_unicode else "|"
+                    return str(0)
+                bar = "\N{CIRCLED TIMES}" if printer._use_unicode else "|"
                 ol = []  # output list, to be concatenated to a string
                 for i, v in enumerate(ar):
                     # if the coef of the dyadic is 1, we skip the 1
                     if ar[i][0] == 1:
-                        ol.extend([u" + ",
+                        ol.extend([" + ",
                                   mpp.doprint(ar[i][1]),
                                   bar,
                                   mpp.doprint(ar[i][2])])
 
                     # if the coef of the dyadic is -1, we skip the 1
                     elif ar[i][0] == -1:
-                        ol.extend([u" - ",
+                        ol.extend([" - ",
                                   mpp.doprint(ar[i][1]),
                                   bar,
                                   mpp.doprint(ar[i][2])])
@@ -232,18 +220,18 @@ class Dyadic(object):
                                 ar[i][0]).parens()[0]
                         else:
                             arg_str = mpp.doprint(ar[i][0])
-                        if arg_str.startswith(u"-"):
+                        if arg_str.startswith("-"):
                             arg_str = arg_str[1:]
-                            str_start = u" - "
+                            str_start = " - "
                         else:
-                            str_start = u" + "
-                        ol.extend([str_start, arg_str, u" ",
+                            str_start = " + "
+                        ol.extend([str_start, arg_str, " ",
                                   mpp.doprint(ar[i][1]),
                                   bar,
                                   mpp.doprint(ar[i][2])])
 
-                outstr = u"".join(ol)
-                if outstr.startswith(u" + "):
+                outstr = "".join(ol)
+                if outstr.startswith(" + "):
                     outstr = outstr[3:]
                 elif outstr.startswith(" "):
                     outstr = outstr[1:]
@@ -309,23 +297,23 @@ class Dyadic(object):
             ol += v[0] * ((other ^ v[1]) | v[2])
         return ol
 
-    def __str__(self, printer=None):
+    def _sympystr(self, printer):
         """Printing method. """
         ar = self.args  # just to shorten things
         if len(ar) == 0:
-            return str(0)
+            return printer._print(0)
         ol = []  # output list, to be concatenated to a string
         for i, v in enumerate(ar):
             # if the coef of the dyadic is 1, we skip the 1
             if ar[i][0] == 1:
-                ol.append(' + (' + str(ar[i][1]) + '|' + str(ar[i][2]) + ')')
+                ol.append(' + (' + printer._print(ar[i][1]) + '|' + printer._print(ar[i][2]) + ')')
             # if the coef of the dyadic is -1, we skip the 1
             elif ar[i][0] == -1:
-                ol.append(' - (' + str(ar[i][1]) + '|' + str(ar[i][2]) + ')')
+                ol.append(' - (' + printer._print(ar[i][1]) + '|' + printer._print(ar[i][2]) + ')')
             # If the coefficient of the dyadic is not 1 or -1,
             # we might wrap it in parentheses, for readability.
             elif ar[i][0] != 0:
-                arg_str = VectorStrPrinter().doprint(ar[i][0])
+                arg_str = printer._print(ar[i][0])
                 if isinstance(ar[i][0], Add):
                     arg_str = "(%s)" % arg_str
                 if arg_str[0] == '-':
@@ -333,8 +321,8 @@ class Dyadic(object):
                     str_start = ' - '
                 else:
                     str_start = ' + '
-                ol.append(str_start + arg_str + '*(' + str(ar[i][1]) +
-                          '|' + str(ar[i][2]) + ')')
+                ol.append(str_start + arg_str + '*(' + printer._print(ar[i][1]) +
+                          '|' + printer._print(ar[i][2]) + ')')
         outstr = ''.join(ol)
         if outstr.startswith(' + '):
             outstr = outstr[3:]
@@ -373,28 +361,6 @@ class Dyadic(object):
             ol += v[0] * (v[1] | (v[2] ^ other))
         return ol
 
-    # We don't define _repr_png_ here because it would add a large amount of
-    # data to any notebook containing SymPy expressions, without adding
-    # anything useful to the notebook. It can still enabled manually, e.g.,
-    # for the qtconsole, with init_printing().
-    def _repr_latex_(self):
-        """
-        IPython/Jupyter LaTeX printing
-
-        To change the behavior of this (e.g., pass in some settings to LaTeX),
-        use init_printing(). init_printing() will also enable LaTeX printing
-        for built in numeric types like ints and container types that contain
-        SymPy objects, like lists and dictionaries of expressions.
-        """
-        from sympy.printing.latex import latex
-        s = latex(self, mode='plain')
-        return "$\\displaystyle %s$" % s
-
-    _repr_latex_orig = _repr_latex_
-
-    _sympystr = __str__
-    _sympyrepr = _sympystr
-    __repr__ = __str__
     __radd__ = __add__
     __rmul__ = __mul__
 
@@ -420,6 +386,8 @@ class Dyadic(object):
         ========
 
         >>> from sympy.physics.vector import ReferenceFrame, outer, dynamicsymbols
+        >>> from sympy.physics.vector import init_vprinting
+        >>> init_vprinting(pretty_print=False)
         >>> N = ReferenceFrame('N')
         >>> q = dynamicsymbols('q')
         >>> B = N.orientnew('B', 'Axis', [q, N.z])
@@ -501,6 +469,8 @@ class Dyadic(object):
         ========
 
         >>> from sympy.physics.vector import ReferenceFrame, outer, dynamicsymbols
+        >>> from sympy.physics.vector import init_vprinting
+        >>> init_vprinting(pretty_print=False)
         >>> N = ReferenceFrame('N')
         >>> q = dynamicsymbols('q')
         >>> B = N.orientnew('B', 'Axis', [q, N.z])
