@@ -1,6 +1,5 @@
 """Implementation of :class:`ExpressionDomain` class. """
 
-from __future__ import print_function, division
 
 from sympy.core import sympify, SympifyError
 from sympy.polys.domains.characteristiczero import CharacteristicZero
@@ -114,20 +113,15 @@ class ExpressionDomain(Field, CharacteristicZero, SimpleDomain):
         def __rtruediv__(f, g):
             return f.simplify(f.__class__(g).ex/f.ex)
 
-        __div__ = __truediv__
-        __rdiv__ = __rtruediv__
-
         def __eq__(f, g):
             return f.ex == f.__class__(g).ex
 
         def __ne__(f, g):
             return not f == g
 
-        def __nonzero__(f):
+        def __bool__(f):
             return f.ex != 0
 
-        __bool__ = __nonzero__
-
         def gcd(f, g):
             from sympy.polys import gcd
             return f.__class__(gcd(f.ex, f.__class__(g).ex))
@@ -173,6 +167,14 @@ class ExpressionDomain(Field, CharacteristicZero, SimpleDomain):
         """Convert a GMPY ``mpq`` object to ``dtype``. """
         return K1(K0.to_sympy(a))
 
+    def from_GaussianIntegerRing(K1, a, K0):
+        """Convert a ``GaussianRational`` object to ``dtype``. """
+        return K1(K0.to_sympy(a))
+
+    def from_GaussianRationalField(K1, a, K0):
+        """Convert a ``GaussianRational`` object to ``dtype``. """
+        return K1(K0.to_sympy(a))
+
     def from_RealField(K1, a, K0):
         """Convert a mpmath ``mpf`` object to ``dtype``. """
         return K1(K0.to_sympy(a))
@@ -222,7 +224,7 @@ class ExpressionDomain(Field, CharacteristicZero, SimpleDomain):
         return a.denom()
 
     def gcd(self, a, b):
-        return a.gcd(b)
+        return self(1)
 
     def lcm(self, a, b):
         return a.lcm(b)
