@@ -1,57 +1,45 @@
 """Computational algebraic field theory. """
 
-from __future__ import print_function, division
 
 from sympy import (
-    S, Rational, AlgebraicNumber,
+    S, Rational, AlgebraicNumber, GoldenRatio, TribonacciConstant,
     Add, Mul, sympify, Dummy, expand_mul, I, pi
 )
-
+from sympy.functions import sqrt, cbrt
+from sympy.core.compatibility import reduce
+from sympy.core.exprtools import Factors
+from sympy.core.function import _mexpand
 from sympy.functions.elementary.exponential import exp
 from sympy.functions.elementary.trigonometric import cos, sin
-
-from sympy.polys.polytools import (
-    Poly, PurePoly, sqf_norm, invert, factor_list, groebner, resultant,
-    degree, poly_from_expr, parallel_poly_from_expr, lcm
-)
-
+from sympy.ntheory import sieve
+from sympy.ntheory.factor_ import divisors
+from sympy.polys.domains import ZZ, QQ
+from sympy.polys.orthopolys import dup_chebyshevt
 from sympy.polys.polyerrors import (
     IsomorphismFailed,
     CoercionFailed,
     NotAlgebraic,
     GeneratorsError,
 )
-
-from sympy.polys.rootoftools import RootOf
-
-from sympy.polys.specialpolys import cyclotomic_poly
-
+from sympy.polys.polytools import (
+    Poly, PurePoly, invert, factor_list, groebner, resultant,
+    degree, poly_from_expr, parallel_poly_from_expr, lcm
+)
 from sympy.polys.polyutils import dict_from_expr, expr_from_dict
-
-from sympy.polys.domains import ZZ, QQ
-
-from sympy.polys.orthopolys import dup_chebyshevt
-
-from sympy.polys.rings import ring
-
 from sympy.polys.ring_series import rs_compose_add
-
+from sympy.polys.rings import ring
+from sympy.polys.rootoftools import CRootOf
+from sympy.polys.specialpolys import cyclotomic_poly
 from sympy.printing.lambdarepr import LambdaPrinter
-
+from sympy.printing.pycode import PythonCodePrinter, MpmathPrinter
+from sympy.simplify.radsimp import _split_gcd
+from sympy.simplify.simplify import _is_sum_surds
 from sympy.utilities import (
     numbered_symbols, variations, lambdify, public, sift
 )
 
-from sympy.core.exprtools import Factors
-from sympy.core.function import _mexpand
-from sympy.simplify.radsimp import _split_gcd
-from sympy.simplify.simplify import _is_sum_surds
-from sympy.ntheory import sieve
-from sympy.ntheory.factor_ import divisors
 from mpmath import pslq, mp
 
-from sympy.core.compatibility import reduce
-from sympy.core.compatibility import range
 
 
 def _choose_factor(factors, x, v, dom=QQ, prec=200, bound=5):
@@ -134,8 +122,8 @@ def _separate_sq(p):
             else:
                 raise NotImplementedError
             continue
-        sifted = sift(y.args, is_sqrt)
-        a.append((Mul(*sifted[False]), Mul(*sifted[True])**2))
+        T, F = sift(y.args, is_sqrt, binary=True)
+        a.append((Mul(*F), Mul(*T)**2))
     a.sort(key=lambda z: z[1])
     if a[-1][1] is S.One:
         # there are no surds
@@ -184,7 +172,6 @@ def _minimal_polynomial_sq(p, n, x):
 
     p = sympify(p)
     n = sympify(n)
-    r = _is_sum_surds(p)
     if not n.is_Integer or not n > 0 or not _is_sum_surds(p):
         return None
     pn = p**Rational(1, n)
@@ -245,9 +232,10 @@ def _minpoly_op_algebraic_element(op, ex1, ex2, x, dom, mp1=None, mp2=None):
     References
     ==========
 
-    [1] http://en.wikipedia.org/wiki/Resultant
-    [2] I.M. Isaacs, Proc. Amer. Math. Soc. 25 (1970), 638
-    "Degrees of sums in a separable field extension".
+    .. [1] https://en.wikipedia.org/wiki/Resultant
+    .. [2] I.M. Isaacs, Proc. Amer. Math. Soc. 25 (1970), 638
+           "Degrees of sums in a separable field extension".
+
     """
     y = Dummy(str(x))
     if mp1 is None:
@@ -466,7 +454,6 @@ def _minpoly_exp(ex, x):
     Returns the minimal polynomial of ``exp(ex)``
     """
     c, a = ex.args[0].as_coeff_Mul()
-    p = sympify(c.p)
     q = sympify(c.q)
     if a == I*pi:
         if c.is_rational:
@@ -500,7 +487,7 @@ def _minpoly_exp(ex, x):
 
 def _minpoly_rootof(ex, x):
     """
-    Returns the minimal polynomial of a ``RootOf`` object.
+    Returns the minimal polynomial of a ``CRootOf`` object.
     """
     p = ex.expr
     p = p.subs({ex.poly.gens[0]:x})
@@ -528,7 +515,24 @@ def _minpoly_compose(ex, x, dom):
     if ex.is_Rational:
         return ex.q*x - ex.p
     if ex is I:
-        return x**2 + 1
+        _, factors = factor_list(x**2 + 1, x, domain=dom)
+        return x**2 + 1 if len(factors) == 1 else x - I
+
+    if ex is GoldenRatio:
+        _, factors = factor_list(x**2 - x - 1, x, domain=dom)
+        if len(factors) == 1:
+            return x**2 - x - 1
+        else:
+            return _choose_factor(factors, x, (1 + sqrt(5))/2, dom=dom)
+
+    if ex is TribonacciConstant:
+        _, factors = factor_list(x**3 - x**2 - x - 1, x, domain=dom)
+        if len(factors) == 1:
+            return x**3 - x**2 - x - 1
+        else:
+            fac = (1 + cbrt(19 - 3*sqrt(33)) + cbrt(19 + 3*sqrt(33))) / 3
+            return _choose_factor(factors, x, fac, dom=dom)
+
     if hasattr(dom, 'symbols') and ex in dom.symbols:
         return x - ex
 
@@ -549,18 +553,21 @@ def _minpoly_compose(ex, x, dom):
         r = sift(f.items(), lambda itx: itx[0].is_Rational and itx[1].is_Rational)
         if r[True] and dom == QQ:
             ex1 = Mul(*[bx**ex for bx, ex in r[False] + r[None]])
-            r1 = r[True]
-            dens = [y.q for _, y in r1]
+            r1 = dict(r[True])
+            dens = [y.q for y in r1.values()]
             lcmdens = reduce(lcm, dens, 1)
-            nums = [base**(y.p*lcmdens // y.q) for base, y in r1]
+            neg1 = S.NegativeOne
+            expn1 = r1.pop(neg1, S.Zero)
+            nums = [base**(y.p*lcmdens // y.q) for base, y in r1.items()]
             ex2 = Mul(*nums)
             mp1 = minimal_polynomial(ex1, x)
             # use the fact that in SymPy canonicalization products of integers
             # raised to rational powers are organized in relatively prime
             # bases, and that in ``base**(n/d)`` a perfect power is
             # simplified with the root
-            mp2 = ex2.q*x**lcmdens - ex2.p
-            ex2 = ex2**Rational(1, lcmdens)
+            # Powers of -1 have to be treated separately to preserve sign.
+            mp2 = ex2.q*x**lcmdens - ex2.p*neg1**(expn1*lcmdens)
+            ex2 = neg1**expn1 * ex2**Rational(1, lcmdens)
             res = _minpoly_op_algebraic_element(Mul, ex1, ex2, x, dom, mp1=mp1, mp2=mp2)
         else:
             res = _minpoly_mul(x, dom, *ex.args)
@@ -572,7 +579,7 @@ def _minpoly_compose(ex, x, dom):
         res = _minpoly_cos(ex, x)
     elif ex.__class__ is exp:
         res = _minpoly_exp(ex, x)
-    elif ex.__class__ is RootOf:
+    elif ex.__class__ is CRootOf:
         res = _minpoly_rootof(ex, x)
     else:
         raise NotAlgebraic("%s doesn't seem to be an algebraic element" % ex)
@@ -580,22 +587,29 @@ def _minpoly_compose(ex, x, dom):
 
 
 @public
-def minimal_polynomial(ex, x=None, **args):
+def minimal_polynomial(ex, x=None, compose=True, polys=False, domain=None):
     """
     Computes the minimal polynomial of an algebraic element.
 
     Parameters
     ==========
 
-    ex : algebraic element expression
-    x : independent variable of the minimal polynomial
+    ex : Expr
+        Element or expression whose minimal polynomial is to be calculated.
 
-    Options
-    =======
+    x : Symbol, optional
+        Independent variable of the minimal polynomial
 
-    compose : if ``True`` ``_minpoly_compose`` is used, if ``False`` the ``groebner`` algorithm
-    polys : if ``True`` returns a ``Poly`` object
-    domain : ground domain
+    compose : boolean, optional (default=True)
+        Method to use for computing minimal polynomial. If ``compose=True``
+        (default) then ``_minpoly_compose`` is used, if ``compose=False`` then
+        groebner bases are used.
+
+    polys : boolean, optional (default=False)
+        If ``True`` returns a ``Poly`` object else an ``Expr`` object.
+
+    domain : Domain, optional
+        Ground domain
 
     Notes
     =====
@@ -630,10 +644,6 @@ def minimal_polynomial(ex, x=None, **args):
     from sympy.polys.domains import FractionField
     from sympy.core.basic import preorder_traversal
 
-    compose = args.get('compose', True)
-    polys = args.get('polys', False)
-    dom = args.get('domain', None)
-
     ex = sympify(ex)
     if ex.is_number:
         # not sure if it's always needed but try it for numbers (issue 8354)
@@ -648,20 +658,24 @@ def minimal_polynomial(ex, x=None, **args):
     else:
         x, cls = Dummy('x'), PurePoly
 
-    if not dom:
-        dom = FractionField(QQ, list(ex.free_symbols)) if ex.free_symbols else QQ
-    if hasattr(dom, 'symbols') and x in dom.symbols:
-        raise GeneratorsError("the variable %s is an element of the ground domain %s" % (x, dom))
+    if not domain:
+        if ex.free_symbols:
+            domain = FractionField(QQ, list(ex.free_symbols))
+        else:
+            domain = QQ
+    if hasattr(domain, 'symbols') and x in domain.symbols:
+        raise GeneratorsError("the variable %s is an element of the ground "
+                              "domain %s" % (x, domain))
 
     if compose:
-        result = _minpoly_compose(ex, x, dom)
+        result = _minpoly_compose(ex, x, domain)
         result = result.primitive()[1]
         c = result.coeff(x**degree(result, x))
         if c.is_negative:
             result = expand_mul(-result)
         return cls(result, x, field=True) if polys else result.collect(x)
 
-    if not dom.is_QQ:
+    if not domain.is_QQ:
         raise NotImplementedError("groebner method only works for QQ")
 
     result = _minpoly_groebner(ex, x, cls)
@@ -686,7 +700,7 @@ def _minpoly_groebner(ex, x, cls):
     from sympy.core.function import expand_multinomial
 
     generator = numbered_symbols('a', cls=Dummy)
-    mapping, symbols, replace = {}, {}, []
+    mapping, symbols = {}, {}
 
     def update_mapping(ex, exp, base=None):
         a = next(generator)
@@ -714,20 +728,15 @@ def _minpoly_groebner(ex, x, cls):
             return Mul(*[ bottom_up_scan(g) for g in ex.args ])
         elif ex.is_Pow:
             if ex.exp.is_Rational:
-                if ex.exp < 0 and ex.base.is_Add:
-                    coeff, terms = ex.base.as_coeff_add()
-                    elt, _ = primitive_element(terms, polys=True)
-
-                    alg = ex.base - coeff
-
-                    # XXX: turn this into eval()
-                    inverse = invert(elt.gen + coeff, elt).as_expr()
-                    base = inverse.subs(elt.gen, alg).expand()
+                if ex.exp < 0:
+                    minpoly_base = _minpoly_groebner(ex.base, x, cls)
+                    inverse = invert(x, minpoly_base).as_expr()
+                    base_inv = inverse.subs(x, ex.base).expand()
 
                     if ex.exp == -1:
-                        return bottom_up_scan(base)
+                        return bottom_up_scan(base_inv)
                     else:
-                        ex = base**(-ex.exp)
+                        ex = base_inv**(-ex.exp)
                 if not ex.exp.is_Integer:
                     base, exp = (
                         ex.base**ex.exp.p).expand(), Rational(1, ex.exp.q)
@@ -759,7 +768,6 @@ def _minpoly_groebner(ex, x, cls):
                     return True
         if ex.is_Mul:
             hit = True
-            a = []
             for p in ex.args:
                 if p.is_Add:
                     return False
@@ -809,16 +817,18 @@ def _minpoly_groebner(ex, x, cls):
 
 
 minpoly = minimal_polynomial
-__all__.append('minpoly')
 
 def _coeffs_generator(n):
     """Generate coefficients for `primitive_element()`. """
-    for coeffs in variations([1, -1], n, repetition=True):
-        yield list(coeffs)
+    for coeffs in variations([1, -1, 2, -2, 3, -3], n, repetition=True):
+        # Two linear combinations with coeffs of opposite signs are
+        # opposites of each other. Hence it suffices to test only one.
+        if coeffs[0] > 0:
+            yield list(coeffs)
 
 
 @public
-def primitive_element(extension, x=None, **args):
+def primitive_element(extension, x=None, *, ex=False, coeffs=_coeffs_generator, polys=False):
     """Construct a common number field for all extensions. """
     if not extension:
         raise ValueError("can't compute primitive element for empty extension")
@@ -827,16 +837,25 @@ def primitive_element(extension, x=None, **args):
         x, cls = sympify(x), Poly
     else:
         x, cls = Dummy('x'), PurePoly
-    if not args.get('ex', False):
-        extension = [ AlgebraicNumber(ext, gen=x) for ext in extension ]
 
-        g, coeffs = extension[0].minpoly.replace(x), [1]
+    coeffs_generator = coeffs
 
+    if not ex:
+        gen, coeffs = extension[0], [1]
+        # XXX when minimal_polynomial is extended to work
+        # with AlgebraicNumbers this test can be removed
+        if isinstance(gen, AlgebraicNumber):
+            g = gen.minpoly.replace(x)
+        else:
+            g = minimal_polynomial(gen, x, polys=True)
         for ext in extension[1:]:
-            s, _, g = sqf_norm(g, x, extension=ext)
-            coeffs = [ s*c for c in coeffs ] + [1]
+            _, factors = factor_list(g, extension=ext)
+            g = _choose_factor(factors, x, gen)
+            s, _, g = g.sqf_norm()
+            gen += s*ext
+            coeffs.append(s)
 
-        if not args.get('polys', False):
+        if not polys:
             return g.as_expr(), coeffs
         else:
             return cls(g), coeffs
@@ -859,8 +878,6 @@ def primitive_element(extension, x=None, **args):
         F.append(f)
         Y.append(y)
 
-    coeffs_generator = args.get('coeffs', _coeffs_generator)
-
     for coeffs in coeffs_generator(len(Y)):
         f = x - sum([ c*y for c, y in zip(coeffs, Y)])
         G = groebner(F + [f], Y + [x], order='lex', field=True)
@@ -880,7 +897,7 @@ def primitive_element(extension, x=None, **args):
 
     _, g = g.clear_denoms()
 
-    if not args.get('polys', False):
+    if not polys:
         return g.as_expr(), coeffs, H
     else:
         return g, coeffs, H
@@ -989,13 +1006,13 @@ def field_isomorphism_factor(a, b):
                 return coeffs
 
             if (a.root + root).evalf(chop=True) == 0:
-                return [ -c for c in coeffs ]
-    else:
-        return None
+                return [-c for c in coeffs]
+
+    return None
 
 
 @public
-def field_isomorphism(a, b, **args):
+def field_isomorphism(a, b, *, fast=True):
     """Construct an isomorphism between two number fields. """
     a, b = sympify(a), sympify(b)
 
@@ -1017,7 +1034,7 @@ def field_isomorphism(a, b, **args):
     if m % n != 0:
         return None
 
-    if args.get('fast', True):
+    if fast:
         try:
             result = field_isomorphism_pslq(a, b)
 
@@ -1030,10 +1047,8 @@ def field_isomorphism(a, b, **args):
 
 
 @public
-def to_number_field(extension, theta=None, **args):
+def to_number_field(extension, theta=None, *, gen=None):
     """Express `extension` in the field generated by `theta`. """
-    gen = args.get('gen')
-
     if hasattr(extension, '__iter__'):
         extension = list(extension)
     else:
@@ -1062,18 +1077,20 @@ def to_number_field(extension, theta=None, **args):
                 "%s is not in a subfield of %s" % (root, theta.root))
 
 
-class IntervalPrinter(LambdaPrinter):
+class IntervalPrinter(MpmathPrinter, LambdaPrinter):
     """Use ``lambda`` printer but print numbers as ``mpi`` intervals. """
 
     def _print_Integer(self, expr):
-        return "mpi('%s')" % super(IntervalPrinter, self)._print_Integer(expr)
+        return "mpi('%s')" % super(PythonCodePrinter, self)._print_Integer(expr)
 
     def _print_Rational(self, expr):
-        return "mpi('%s')" % super(IntervalPrinter, self)._print_Rational(expr)
+        return "mpi('%s')" % super(PythonCodePrinter, self)._print_Rational(expr)
+
+    def _print_Half(self, expr):
+        return "mpi('%s')" % super(PythonCodePrinter, self)._print_Rational(expr)
 
     def _print_Pow(self, expr):
-        return super(IntervalPrinter, self)._print_Pow(expr, rational=True)
-
+        return super(MpmathPrinter, self)._print_Pow(expr, rational=True)
 
 @public
 def isolate(alg, eps=None, fast=False):
