@@ -11,13 +11,77 @@ try:
 except ImportError:
     pass
 
-from sympy.core.compatibility import unicode, u_decode
 from sympy.utilities.decorator import doctest_depends_on
-from sympy.utilities.misc import find_executable
 from .latex import latex
 
 __doctest_requires__ = {('preview',): ['pyglet']}
 
+
+def _check_output_no_window(*args, **kwargs):
+    # Avoid showing a cmd.exe window when running this
+    # on Windows
+    if os.name == 'nt':
+        creation_flag = 0x08000000 # CREATE_NO_WINDOW
+    else:
+        creation_flag = 0 # Default value
+    return check_output(*args, creationflags=creation_flag, **kwargs)
+
+
+def _run_pyglet(fname, fmt):
+    from pyglet import window, image, gl
+    from pyglet.window import key
+
+    if fmt == "png":
+        from pyglet.image.codecs.png import PNGImageDecoder
+        img = image.load(fname, decoder=PNGImageDecoder())
+    else:
+        raise ValueError("pyglet preview works only for 'png' files.")
+
+    offset = 25
+
+    config = gl.Config(double_buffer=False)
+    win = window.Window(
+        width=img.width + 2*offset,
+        height=img.height + 2*offset,
+        caption="sympy",
+        resizable=False,
+        config=config
+    )
+
+    win.set_vsync(False)
+
+    try:
+        def on_close():
+            win.has_exit = True
+
+        win.on_close = on_close
+
+        def on_key_press(symbol, modifiers):
+            if symbol in [key.Q, key.ESCAPE]:
+                on_close()
+
+        win.on_key_press = on_key_press
+
+        def on_expose():
+            gl.glClearColor(1.0, 1.0, 1.0, 1.0)
+            gl.glClear(gl.GL_COLOR_BUFFER_BIT)
+
+            img.blit(
+                (win.width - img.width) / 2,
+                (win.height - img.height) / 2
+            )
+
+        win.on_expose = on_expose
+
+        while not win.has_exit:
+            win.dispatch_events()
+            win.flip()
+    except KeyboardInterrupt:
+        pass
+
+    win.close()
+
+
 @doctest_depends_on(exe=('latex', 'dvipng'), modules=('pyglet',),
             disable_viewers=('evince', 'gimp', 'superior-dvi-viewer'))
 def preview(expr, output='png', viewer=None, euler=True, packages=(),
@@ -124,16 +188,18 @@ def preview(expr, output='png', viewer=None, euler=True, packages=(),
             }
 
             try:
-                for candidate in candidates[output]:
-                    path = find_executable(candidate)
-                    if path is not None:
-                        viewer = path
-                        break
-                else:
-                    raise SystemError(
-                        "No viewers found for '%s' output format." % output)
+                candidate_viewers = candidates[output]
             except KeyError:
-                raise SystemError("Invalid output format: %s" % output)
+                raise ValueError("Invalid output format: %s" % output) from None
+
+            for candidate in candidate_viewers:
+                path = shutil.which(candidate)
+                if path is not None:
+                    viewer = path
+                    break
+            else:
+                raise OSError(
+                    "No viewers found for '%s' output format." % output)
     else:
         if viewer == "file":
             if filename is None:
@@ -142,8 +208,8 @@ def preview(expr, output='png', viewer=None, euler=True, packages=(),
             if outputbuffer is None:
                 raise ValueError("outputbuffer has to be a BytesIO "
                                  "compatible object if viewer=\"BytesIO\"")
-        elif viewer not in special and not find_executable(viewer):
-            raise SystemError("Unrecognized viewer: %s" % viewer)
+        elif viewer not in special and not shutil.which(viewer):
+            raise OSError("Unrecognized viewer: %s" % viewer)
 
 
     if preamble is None:
@@ -162,7 +228,6 @@ def preview(expr, output='png', viewer=None, euler=True, packages=(),
         if packages:
             raise ValueError("The \"packages\" keyword must not be set if a "
                              "custom LaTeX preamble was specified")
-    latex_main = preamble + '\n%s\n\n' + r"\end{document}"
 
     if isinstance(expr, str):
         latex_string = expr
@@ -171,80 +236,83 @@ def preview(expr, output='png', viewer=None, euler=True, packages=(),
                         latex(expr, mode='plain', **latex_settings) +
                         '$')
 
-    try:
-        workdir = tempfile.mkdtemp()
+    latex_main = preamble + '\n' + latex_string + '\n\n' + r"\end{document}"
 
+    with tempfile.TemporaryDirectory() as workdir:
         with io.open(join(workdir, 'texput.tex'), 'w', encoding='utf-8') as fh:
-            fh.write(unicode(latex_main) % u_decode(latex_string))
+            fh.write(latex_main)
 
         if outputTexFile is not None:
             shutil.copyfile(join(workdir, 'texput.tex'), outputTexFile)
 
-        if not find_executable('latex'):
+        if not shutil.which('latex'):
             raise RuntimeError("latex program is not installed")
 
         try:
-            # Avoid showing a cmd.exe window when running this
-            # on Windows
-            if os.name == 'nt':
-                creation_flag = 0x08000000 # CREATE_NO_WINDOW
-            else:
-                creation_flag = 0 # Default value
-            check_output(['latex', '-halt-on-error', '-interaction=nonstopmode',
-                          'texput.tex'],
-                         cwd=workdir,
-                         stderr=STDOUT,
-                         creationflags=creation_flag)
+            _check_output_no_window(
+                ['latex', '-halt-on-error', '-interaction=nonstopmode',
+                 'texput.tex'],
+                cwd=workdir,
+                stderr=STDOUT)
         except CalledProcessError as e:
             raise RuntimeError(
                 "'latex' exited abnormally with the following output:\n%s" %
                 e.output)
 
+        src = "texput.%s" % (output)
+
         if output != "dvi":
+            # in order of preference
+            commandnames = {
+                "ps": ["dvips"],
+                "pdf": ["dvipdfmx", "dvipdfm", "dvipdf"],
+                "png": ["dvipng"],
+                "svg": ["dvisvgm"],
+            }
+            try:
+                cmd_variants = commandnames[output]
+            except KeyError:
+                raise ValueError("Invalid output format: %s" % output) from None
+
+            # find an appropriate command
+            for cmd_variant in cmd_variants:
+                cmd_path = shutil.which(cmd_variant)
+                if cmd_path:
+                    cmd = [cmd_path]
+                    break
+            else:
+                if len(cmd_variants) > 1:
+                    raise RuntimeError("None of %s are installed" % ", ".join(cmd_variants))
+                else:
+                    raise RuntimeError("%s is not installed" % cmd_variants[0])
+
             defaultoptions = {
-                "ps": [],
-                "pdf": [],
-                "png": ["-T", "tight", "-z", "9", "--truecolor"],
-                "svg": ["--no-fonts"],
+                "dvipng": ["-T", "tight", "-z", "9", "--truecolor"],
+                "dvisvgm": ["--no-fonts"],
             }
 
             commandend = {
-                "ps": ["-o", "texput.ps", "texput.dvi"],
-                "pdf": ["texput.dvi", "texput.pdf"],
-                "png": ["-o", "texput.png", "texput.dvi"],
-                "svg": ["-o", "texput.svg", "texput.dvi"],
+                "dvips": ["-o", src, "texput.dvi"],
+                "dvipdf": ["texput.dvi", src],
+                "dvipdfm": ["-o", src, "texput.dvi"],
+                "dvipdfmx": ["-o", src, "texput.dvi"],
+                "dvipng": ["-o", src, "texput.dvi"],
+                "dvisvgm": ["-o", src, "texput.dvi"],
             }
 
-            if output == "svg":
-                cmd = ["dvisvgm"]
+            if dvioptions is not None:
+                cmd.extend(dvioptions)
             else:
-                cmd = ["dvi" + output]
-            if not find_executable(cmd[0]):
-                raise RuntimeError("%s is not installed" % cmd[0])
-            try:
-                if dvioptions is not None:
-                    cmd.extend(dvioptions)
-                else:
-                    cmd.extend(defaultoptions[output])
-                cmd.extend(commandend[output])
-            except KeyError:
-                raise SystemError("Invalid output format: %s" % output)
+                cmd.extend(defaultoptions.get(cmd_variant, []))
+            cmd.extend(commandend[cmd_variant])
 
             try:
-                # Avoid showing a cmd.exe window when running this
-                # on Windows
-                if os.name == 'nt':
-                    creation_flag = 0x08000000 # CREATE_NO_WINDOW
-                else:
-                    creation_flag = 0 # Default value
-                check_output(cmd, cwd=workdir, stderr=STDOUT,
-                             creationflags=creation_flag)
+                _check_output_no_window(cmd, cwd=workdir, stderr=STDOUT)
             except CalledProcessError as e:
                 raise RuntimeError(
                     "'%s' exited abnormally with the following output:\n%s" %
                     (' '.join(cmd), e.output))
 
-        src = "texput.%s" % (output)
 
         if viewer == "file":
             shutil.move(join(workdir, src), filename)
@@ -253,77 +321,16 @@ def preview(expr, output='png', viewer=None, euler=True, packages=(),
                 outputbuffer.write(fh.read())
         elif viewer == "pyglet":
             try:
-                from pyglet import window, image, gl
-                from pyglet.window import key
+                import pyglet  # noqa: F401
             except ImportError:
                 raise ImportError("pyglet is required for preview.\n visit http://www.pyglet.org/")
 
-            if output == "png":
-                from pyglet.image.codecs.png import PNGImageDecoder
-                img = image.load(join(workdir, src), decoder=PNGImageDecoder())
-            else:
-                raise SystemError("pyglet preview works only for 'png' files.")
-
-            offset = 25
-
-            config = gl.Config(double_buffer=False)
-            win = window.Window(
-                width=img.width + 2*offset,
-                height=img.height + 2*offset,
-                caption="sympy",
-                resizable=False,
-                config=config
-            )
-
-            win.set_vsync(False)
-
-            try:
-                def on_close():
-                    win.has_exit = True
-
-                win.on_close = on_close
-
-                def on_key_press(symbol, modifiers):
-                    if symbol in [key.Q, key.ESCAPE]:
-                        on_close()
-
-                win.on_key_press = on_key_press
-
-                def on_expose():
-                    gl.glClearColor(1.0, 1.0, 1.0, 1.0)
-                    gl.glClear(gl.GL_COLOR_BUFFER_BIT)
-
-                    img.blit(
-                        (win.width - img.width) / 2,
-                        (win.height - img.height) / 2
-                    )
-
-                win.on_expose = on_expose
-
-                while not win.has_exit:
-                    win.dispatch_events()
-                    win.flip()
-            except KeyboardInterrupt:
-                pass
-
-            win.close()
+            return _run_pyglet(join(workdir, src), fmt=output)
         else:
             try:
-                # Avoid showing a cmd.exe window when running this
-                # on Windows
-                if os.name == 'nt':
-                    creation_flag = 0x08000000 # CREATE_NO_WINDOW
-                else:
-                    creation_flag = 0 # Default value
-                check_output([viewer, src], cwd=workdir, stderr=STDOUT,
-                             creationflags=creation_flag)
+                _check_output_no_window(
+                    [viewer, src], cwd=workdir, stderr=STDOUT)
             except CalledProcessError as e:
                 raise RuntimeError(
                     "'%s %s' exited abnormally with the following output:\n%s" %
                     (viewer, src, e.output))
-    finally:
-        try:
-            shutil.rmtree(workdir) # delete directory
-        except OSError as e:
-            if e.errno != 2: # code 2 - no such file or directory
-                raise
